@@ -14,11 +14,14 @@ from collective.documentviewer.utils import allowedDocumentType
 from collective.iconifiedcategory import utils
 from collective.iconifiedcategory.content.subcategory import ISubcategory
 from collective.iconifiedcategory.interfaces import IIconifiedPreview
+from collective.iconifiedcategory.utils import _modified
 from plone import api
 from plone.app.contenttypes.interfaces import IFile
 from plone.app.contenttypes.interfaces import IImage
 from plone.app.contenttypes.interfaces import ILink
 from plone.indexer.interfaces import IIndexableObject
+from Products.CMFCore.permissions import View
+from Products.CMFCore.utils import _checkPermission
 from zope.annotation import IAnnotations
 from zope.component import queryMultiAdapter
 
@@ -38,6 +41,8 @@ class CategorizedObjectInfoAdapter(object):
             'subcategory_uid': None,
             'subcategory_id': None,
             'subcategory_title': None,
+            # this will be useable for caching for example
+            'last_updated': self._last_updated,
             'icon_url': utils.get_category_icon_url(category),
             'to_be_printed_activated': self._to_be_printed_activated(category),
             'confidentiality_activated': self._confidentiality_activated(category),
@@ -102,6 +107,12 @@ class CategorizedObjectInfoAdapter(object):
         if IImage.providedBy(self.obj):
             return self.obj.image.size
 
+    @property
+    def _last_updated(self):
+        """When updating some attributes directly or annotation,
+           "modified" is not changed."""
+        return _modified(self.context)
+
     def _to_be_printed_activated(self, category):
         category_group = category.get_category_group()
         return category_group.to_be_printed_activated
@@ -185,7 +196,11 @@ class CategorizedObjectAdapter(object):
         self.categorized_obj = categorized_obj
 
     def can_view(self):
-        return True
+        """By default, check that current user may View the context.
+           Indeed for some advanced management (@@download), views are
+           declared permission="zope2.Public"."""
+        if _checkPermission(View, self.context):
+            return True
 
 
 class CategorizedObjectPreviewAdapter(object):
