@@ -7,9 +7,8 @@ Created by mpeeters
 :license: GPL, see LICENCE.txt for more details.
 """
 
-from Products.Five import BrowserView
-
 from collective.iconifiedcategory import utils
+from Products.Five import BrowserView
 
 
 class IconifiedCategory(BrowserView):
@@ -22,7 +21,10 @@ class IconifiedCategory(BrowserView):
                u"background-size: contain; }}")
         if utils.has_config_root(self.context) is False:
             return ''
-        categories = utils.get_categories(self.context)
+        # sort_on=None to avoid useless sort_on="getObjPositionInParent"
+        categories = utils.get_categories(self.context,
+                                          sort_on=None,
+                                          only_enabled=False)
         for category in categories:
             obj = category._unrestrictedGetObject()
             category_id = utils.calculate_category_id(obj)
