@@ -32,20 +32,22 @@ class CategorizedTabView(BrowserView):
 
     def table_render(self, portal_type=None):
         self.portal_type = portal_type
-        table = CategorizedTable(self.context, self.request, portal_type=portal_type)
-        self._prepare_table_render(table)
-        table.update()
-        return table.render()
+        table = self.context.restrictedTraverse('@@iconifiedcategory_table')
+        self.table = table
+        self.table.portal_type = portal_type
+        self._prepare_table_render()
+        self.table.update()
+        return self.table.render()
 
-    def _prepare_table_render(self, table, **kwargs):
+    def _prepare_table_render(self, **kwargs):
         if self.show('confidentiality'):
-            alsoProvides(table, ICategorizedConfidential)
+            alsoProvides(self.table, ICategorizedConfidential)
         if self.show('to_be_printed'):
-            alsoProvides(table, ICategorizedPrint)
+            alsoProvides(self.table, ICategorizedPrint)
         if self.show('publishable'):
-            alsoProvides(table, ICategorizedPublishable)
+            alsoProvides(self.table, ICategorizedPublishable)
         if self.show('signed'):
-            alsoProvides(table, ICategorizedSigned)
+            alsoProvides(self.table, ICategorizedSigned)
 
     def _config(self):
         """ """
@@ -120,7 +122,7 @@ class CategorizedContent(object):
         return portal_url + '/' + suffix
 
 
-class CategorizedTable(Table):
+class CategorizedTable(Table, BrowserView):
     implements(ICategorizedTable)
 
     cssClasses = {'table': 'listing iconified-listing nosort'}
@@ -243,6 +245,14 @@ class FilesizeColumn(column.GetAttrColumn):
     header = _(u'Filesize')
     weight = 70
 
+    def renderHeadCell(self):
+        """ """
+        header = super(FilesizeColumn, self).renderHeadCell()
+        total = sum([v.filesize for v in self.table.values
+                     if v.filesize is not None])
+        header += u"<p>(Total: {0})</p>".format(utils.render_filesize(total))
+        return header
+
     def renderCell(self, content):
         if getattr(content, 'filesize', None) is None:
             return ''
