@@ -18,34 +18,46 @@ from plone.namedfile.browser import DisplayFile
 from plone.namedfile.browser import Download
 from plone.namedfile.scaling import ImageScaling
 from plone.rfc822.interfaces import IPrimaryFieldInfo
+from Products.CMFCore.permissions import View
+from Products.CMFCore.utils import _checkPermission
 from Products.Five import BrowserView
 from zope.component import getMultiAdapter
 from zope.component.interfaces import ComponentLookupError
 
+import json
+
 
 class CategorizedChildView(BrowserView):
     """ """
-    def __init__(self, context, request):
-        """ """
-        super(CategorizedChildView, self).__init__(context, request)
-        self.portal_url = api.portal.get().absolute_url()
+
+    @property
+    def _filters(self):
+        """Overridable method to define custom filters."""
+        return {}
+
+    def _filters_json(self):
+        """Filters are stored in template as json."""
+        return json.dumps(self._filters)
 
     def update(self):
         self.categorized_elements = get_categorized_elements(
             self.context,
             portal_type=self.portal_type,
+            filters=self._filters,
+            check_can_view=self.check_can_view,
         )
 
-    def __call__(self, portal_type=None, show_nothing=True):
-        """ """
+    def __call__(self, portal_type=None, show_nothing=True, check_can_view=False):
+        """We set p_check_can_view=False by default."""
+        self.portal_url = api.portal.get().absolute_url()
         self.portal_type = portal_type
         self.show_nothing = show_nothing
+        self.check_can_view = check_can_view
         self.update()
         return super(CategorizedChildView, self).__call__()
 
     def has_elements_to_show(self):
-        return ('categorized_elements' in self.context.__dict__ and
-                len(self.categorized_elements) > 0)
+        return ('categorized_elements' in self.context.__dict__ and len(self.categorized_elements) > 0)
 
     def categories_infos(self):
         infos = [(e['category_uid'], {'id': e['category_id'],
@@ -67,27 +79,26 @@ class CategorizedChildInfosView(BrowserView):
         """ """
         super(CategorizedChildInfosView, self).__init__(context, request)
         self.portal_url = api.portal.get().absolute_url()
-        self.have_details_to_show = False
 
     def update(self):
-        uids = self._find_uids()
-        self.categorized_elements = get_categorized_elements(self.context,
-                                                             uids=uids)
+        self.filters['category_uid'] = self.category_uid
+        self.categorized_elements = get_categorized_elements(
+            self.context,
+            filters=self.filters)
+        self._infos = self.infos()
 
-    def _find_uids(self):
-        """ """
-        uids = []
-        for k, v in getattr(self.context, 'categorized_elements', {}).items():
-            if v['category_uid'] == self.category_uid:
-                uids.append(k)
-        return uids
-
-    def __call__(self, category_uid):
+    def __call__(self, category_uid, filters):
         """ """
         self.category_uid = category_uid
+        self.filters = filters
         self.update()
         return super(CategorizedChildInfosView, self).__call__()
 
+    def show_details(self, number_of_columns):
+        """Only show details if displaying at most 2 columns of elements
+           in the tooltispter popup."""
+        return bool(number_of_columns < 3)
+
     def show_preview_link(self):
         """Made to be overrided."""
         return True
@@ -102,12 +113,21 @@ class CategorizedChildInfosView(BrowserView):
         infos = OrderedDict([(e, []) for e in self.categories_uids])
         for element in self.categorized_elements:
             infos[element['category_uid']].append(element)
+        self._infos = infos
         return infos
 
     def render_filesize(self, size):
         """ """
         return render_filesize(size)
 
+    def show_more_infos_url(self):
+        """
+        In some case we show categorized content but user does not have
+        access to the context, in the case do not show the more infos link.
+        """
+        if _checkPermission(View, self.context):
+            return True
+
     def categorized_elements_more_infos_url(self):
         """ """
         return "{0}/{1}".format(self.context.absolute_url(), "@@iconifiedcategory")
@@ -125,18 +145,12 @@ class CategorizedChildInfosView(BrowserView):
     def show(self, element, attr_prefix):
         """ """
         show = element['{0}_activated'.format(attr_prefix)] and self._show_detail(attr_prefix)
-        if show:
-            self.have_details_to_show = True
         return show
 
     def _show_detail(self, detail_type):
         """Made to be overrided."""
         return True
 
-    def show_details_action(self):
-        """ """
-        return self.have_details_to_show
-
     def get_css_classses_for(self, functionnality, element):
         """ """
         css_classes = []
@@ -189,7 +203,9 @@ class CanViewAwareDownload(Download):
     def __call__(self):
         if not check_can_view(self.context, self.request):
             raise Unauthorized
-        return super(CanViewAwareDownload, self).__call__()
+        # access is managed by can_view
+        with api.env.adopt_roles(['Manager']):
+            return super(CanViewAwareDownload, self).__call__()
 
 
 class CanViewAwareDisplayFile(DisplayFile, CanViewAwareDownload):
@@ -201,7 +217,9 @@ class CanViewAwareFNWDownload(fnw_Download):
     def __call__(self):
         if not check_can_view(aq_inner(self.context.context), self.request):
             raise Unauthorized
-        return super(CanViewAwareFNWDownload, self).__call__()
+        # access is managed by can_view
+        with api.env.adopt_roles(['Manager']):
+            return super(CanViewAwareFNWDownload, self).__call__()
 
 
 class ImageDataModifiedImageScaling(ImageScaling):
