@@ -14,6 +14,7 @@ from collective.iconifiedcategory.tests.base import BaseTestCase
 from plone import api
 from plone.dexterity.utils import createContentInContainer
 from zExceptions import Redirect
+
 import transaction
 
 
@@ -287,10 +288,11 @@ class TestUtils(BaseTestCase):
                          '1 KB')
         self.assertEqual(utils.render_filesize(5000),
                          '4 KB')
+        # soft warning if filesize no more in B/KB
+        self.assertEqual(utils.render_filesize(2500000),
+                         u"<span class='soft_warn_filesize'>2.4 MB</span>")
         self.assertEqual(utils.render_filesize(5000000),
-                         '4.8 MB')
-        self.assertEqual(utils.render_filesize(5000000),
-                         '4.8 MB')
+                         u"<span class='soft_warn_filesize'>4.8 MB</span>")
         # warning if filesize > 5000000
         self.assertEqual(utils.render_filesize(5000001),
                          u"<span class='warn_filesize' title='Annex size is huge, "
@@ -314,6 +316,8 @@ class TestUtils(BaseTestCase):
             confidential=False,
         )
         scale = category.restrictedTraverse('@@images').scale(scale='listing').__name__
+        # while creating, modified is changed again after categorized_update
+        utils.update_all_categorized_elements(self.portal)
         result = utils.get_categorized_elements(self.portal)
         self.assertEqual(
             result,
@@ -329,6 +333,7 @@ class TestUtils(BaseTestCase):
               'filesize': None,
               'icon_url': u'config/group-1/category-x/@@images/{0}'.format(scale),
               'id': 'doc-subcategory-move',
+              'last_updated': self._modified(document),
               'portal_type': 'Document',
               'preview_status': 'not_convertable',
               'publishable': False,
@@ -404,6 +409,23 @@ class TestUtils(BaseTestCase):
         api.content.delete(document2)
         api.content.delete(category)
 
+    def test_get_categorized_elements_filters(self):
+        self.config.get('group-1').confidentiality_activated = True
+        createContentInContainer(
+            container=self.portal,
+            portal_type='Document',
+            title='Doc1',
+            content_category='config_-_group-1_-_category-1-1',
+            to_print=False,
+            confidential=True,
+        )
+        self.assertEqual(len(utils.get_categorized_elements(
+            self.portal, filters={'confidential': True})),
+            1)
+        self.assertEqual(len(utils.get_categorized_elements(
+            self.portal, filters={'confidential': False})),
+            0)
+
     def test_update_categorized_elements(self):
         document2 = createContentInContainer(
             container=self.portal,
@@ -526,3 +548,23 @@ class TestUtils(BaseTestCase):
         category = subcategory.get_category()
         scale = category.restrictedTraverse('@@images').scale(scale='listing').__name__
         self.assertEqual(doc_icon_url, u'config/group-1/category-x/@@images/{0}'.format(scale))
+
+    def test_get_ordered_categories(self):
+        """Test that caching is behaving correctly."""
+        # invalidated when new category added
+        category = api.content.create(
+            type='ContentCategory',
+            title='Category X',
+            icon=self.icon,
+            container=self.config['group-1'],
+        )
+        category_uid = category.UID()
+        self.assertTrue(category_uid in utils.get_ordered_categories(self.portal))
+        # invalidated when category deleted
+        api.content.delete(category)
+        self.assertFalse(category_uid in utils.get_ordered_categories(self.portal))
+        # invalidated when a category position changed
+        res = sorted(utils.get_ordered_categories(self.portal).items())
+        self.portal.config.get('group-1').moveObjectsDown('category-1-2')
+        res_after_cat_position_changed = sorted(utils.get_ordered_categories(self.portal).items())
+        self.assertNotEqual(res, res_after_cat_position_changed)
