@@ -21,6 +21,9 @@ from collective.iconifiedcategory.interfaces import IIconifiedCategoryGroup
 from collective.iconifiedcategory.interfaces import IIconifiedCategorySettings
 from collective.iconifiedcategory.interfaces import IIconifiedContent
 from collective.iconifiedcategory.interfaces import IIconifiedInfos
+from datetime import datetime
+from imio.helpers.cache import get_cachekey_volatile
+from imio.helpers.content import find
 from natsort import natsorted
 from plone import api
 from plone.app.contenttypes.interfaces import IFile
@@ -28,6 +31,7 @@ from plone.app.contenttypes.interfaces import IImage
 from plone.memoize import ram
 from Products.CMFPlone.utils import safe_unicode
 from time import time
+from zope.annotation import IAnnotations
 from zope.component import getAdapter
 from zope.component import getMultiAdapter
 from zope.component import queryAdapter
@@ -51,7 +55,7 @@ def query_config_root(context):
         query = {
             'portal_type': 'ContentCategoryConfiguration',
         }
-        result = catalog.unrestrictedSearchResults(query)
+        result = catalog.unrestrictedSearchResults(**query)
         if not result:
             return
         config_root = result[0]._unrestrictedGetObject()
@@ -87,12 +91,17 @@ def get_categories(context,
     catalog = api.portal.get_tool('portal_catalog')
     query = {
         'object_provides': 'collective.iconifiedcategory.content.category.ICategory',
-        'sort_on': sort_on,
-        'path': '/'.join(config_group.getPhysicalPath()),
     }
+    # query on path is context is not the Plone Site
+    # happens when computing categories to generate CSS
+    if context.portal_type != "Plone Site":
+        query['path'] = '/'.join(config_group.getPhysicalPath())
     if only_enabled:
         query['enabled'] = True
-    res = catalog.unrestrictedSearchResults(query)
+    if sort_on:
+        query['sort_on'] = sort_on
+
+    res = catalog.unrestrictedSearchResults(**query)
     if the_objects:
         res = [brain.getObject() for brain in res]
     return res
@@ -203,11 +212,7 @@ def update_all_categorized_elements(container, limited=False, sort=True):
 
 def get_ordered_categories_cachekey(method, context, only_enabled=True):
     """ """
-    # while using plone.app.async, there is not REQUEST
-    debug = 'no_request'
-    if hasattr(context, 'REQUEST'):
-        debug = str(context.REQUEST._debug)
-    return debug, get_config_root(context), only_enabled
+    return get_cachekey_volatile("collective.iconifiedcategory.utils.get_ordered_categories")
 
 
 @ram.cache(get_ordered_categories_cachekey)
@@ -224,7 +229,7 @@ def get_ordered_categories(context, only_enabled=True):
     for idx, category in enumerate(categories):
         elements[category.UID] = idx
         elements[calculate_category_id(category.getObject())] = idx
-        subcategories = api.content.find(context=category, **query)
+        subcategories = find(context=category, unrestricted=True, **query)
         for subcategory in subcategories:
             elements[subcategory.UID] = idx
             elements[calculate_category_id(subcategory.getObject())] = idx
@@ -264,64 +269,111 @@ def _categorized_elements(context):
     )
 
 
+def _check_filters(infos, filters):
+    """ """
+    keep = True
+    for k, v in filters.items():
+        # manage case when stored value is a list or not
+        stored_value = infos[k]
+        if not hasattr(stored_value, '__iter__'):
+            stored_value = (stored_value, )
+        if v not in stored_value:
+            keep = False
+            break
+    return keep
+
+
+def _get_adapter(context, obj, adapter):
+    """ """
+    if adapter is None:
+        adapter = getMultiAdapter(
+            (context, context.REQUEST, obj),
+            IIconifiedContent)
+    else:
+        adapter.categorized_obj = obj
+    return adapter
+
+
 def get_categorized_elements(context,
                              result_type='dict',
                              portal_type=None,
                              sort_on=None,
-                             uids=[]):
+                             uids=[],
+                             filters={},
+                             check_can_view=True,
+                             caching=True):
     """Return categorized elements.
        p_result_type may be :
        - 'dict': default, essential metadata are returned as a dict;
-       - 'objects': categorized objects are returned."""
-    elements = []
-    categorized_elements = _categorized_elements(context)
-    if not categorized_elements:
-        return elements
+       - 'objects': categorized objects are returned.
+       If some p_filters are given, the values will be filtered,
+       available filters are values stored in categorized_elements.
+       If p_check_can_view is True, then the IIconifiedContent.can_view
+       check will be called."""
+    elements = None
+    if caching:
+        # in some cases like in tests, request can not be retrieved
+        key = "collective.iconifiedcategory.get_categorized_elements'" \
+            "-{0}-{1}-{2}-{3}-{4}".format(
+                repr(context),
+                result_type,
+                portal_type,
+                '_'.join(uids),
+                ['{0}_{1}'.format(k, v) for k, v in filters.items()])
+        cache = IAnnotations(context.REQUEST)
+        elements = cache.get(key, None)
 
-    uids = uids or categorized_elements.keys()
-    catalog = api.portal.get_tool('portal_catalog')
-    current_user_allowedRolesAndUsers = catalog._listAllowedRolesAndUsers(api.user.get_current())
-    for uid, infos in categorized_elements.items():
-        if uids and uid not in uids or \
-           portal_type and infos['portal_type'] != portal_type or \
-           (infos['confidential'] and
-                not set(infos['allowedRolesAndUsers']).intersection(current_user_allowedRolesAndUsers)):
-            continue
-        obj = context.get(infos['id'])
-        obj_uid = obj.UID()
-        adapter = getMultiAdapter(
-            (obj.aq_parent, obj.REQUEST, obj),
-            IIconifiedContent)
-        if adapter.can_view():
-            if result_type == 'objects':
-                elements.append(obj)
+    if elements is None:
+        elements = []
+        categorized_elements = _categorized_elements(context)
+        if not categorized_elements:
+            return elements
+
+        uids = uids or categorized_elements.keys()
+        catalog = api.portal.get_tool('portal_catalog')
+        current_user_allowedRolesAndUsers = catalog._listAllowedRolesAndUsers(api.user.get_current())
+        adapter = None
+        for uid, infos in categorized_elements.items():
+            if (uids and uid not in uids) or \
+               (portal_type and infos['portal_type'] != portal_type) or \
+               not _check_filters(infos, filters) or \
+               (infos['confidential'] and not set(infos['allowedRolesAndUsers'])
+                   .intersection(current_user_allowedRolesAndUsers)):
+                continue
+
+            obj = context.get(infos['id'])
+            adapter = _get_adapter(context, obj, adapter)
+            if not check_can_view or adapter.can_view():
+                if result_type == 'objects':
+                    elements.append(obj)
+                else:
+                    # add 'UID' to the available infos
+                    tmp = categorized_elements[uid].copy()
+                    tmp['UID'] = uid
+                    elements.append(tmp)
+
+        if elements and sort_on:
+            if result_type == 'dict':
+                if sort_on in elements[0]:
+                    elements = sorted(elements, key=lambda x, sort_on=sort_on: x[sort_on])
+                elif sort_on == 'getObjPositionInParent':
+                    elements = sorted(
+                        elements,
+                        key=lambda x, object_ids=context.objectIds(): object_ids.index(x['id']))
             else:
-                # add 'UID' to the available infos
-                tmp = categorized_elements[obj_uid].copy()
-                tmp['UID'] = obj_uid
-                elements.append(tmp)
+                if getattr(elements[0], sort_on):
+                    elements = sorted(elements, key=lambda x, sort_on=sort_on: getattr(x, sort_on))
+                elif sort_on == 'getObjPositionInParent':
+                    elements = sorted(
+                        elements,
+                        key=lambda x, object_ids=context.objectIds(): object_ids.index(x.id))
 
-    if elements and sort_on:
-        if result_type == 'dict':
-            if sort_on in elements[0]:
-                elements = sorted(elements, key=lambda x, sort_on=sort_on: x[sort_on])
-            elif sort_on == 'getObjPositionInParent':
-                elements = sorted(
-                    elements,
-                    key=lambda x, object_ids=context.objectIds(): object_ids.index(x['id']))
-        else:
-            if getattr(elements[0], sort_on):
-                elements = sorted(elements, key=lambda x, sort_on=sort_on: getattr(x, sort_on))
-            elif sort_on == 'getObjPositionInParent':
-                elements = sorted(
-                    elements,
-                    key=lambda x, object_ids=context.objectIds(): object_ids.index(x.id))
     return elements
 
 
 def get_back_references(obj):
     catalog = api.portal.get_tool('portal_catalog')
-    brains = catalog(content_category_uid=obj.UID())
+    brains = catalog.unrestrictedSearchResults(content_category_uid=obj.UID())
     return brains
 
 
@@ -376,6 +428,9 @@ def render_filesize(size):
                           default='Annex size is huge, it could '
                           'be difficult to be downloaded!'),
                 pretty_filesize)
+    elif not pretty_filesize.endswith((' B', ' KB')):
+        pretty_filesize = \
+            u"<span class='soft_warn_filesize'>{0}</span>".format(pretty_filesize)
     return pretty_filesize
 
 
@@ -456,3 +511,13 @@ def validateFileIsPDF(data):
             category = get_category_object(context, data.content_category)
             if category.only_pdf:
                 raise Invalid(_(u"You must select a PDF file!"))
+
+
+def _modified(obj, asdatetime=True):
+    """Returns max value between obj.modified() and obj._p_mtime,
+       in case an annotation is changed on obj, obj._p_mtime is changed,
+       not obj.modified()."""
+    modified = max(float(obj.modified()), obj._p_mtime)
+    if asdatetime:
+        modified = datetime.fromtimestamp(modified)
+    return modified
